@@ -85,6 +85,20 @@ class glidepath:
             pt[:,i,:] = np.vstack([np.ones(ports_cum_ret.shape[1]), np.exp(ports_cum_ret)])
         return pt, ports_rt
 
+    def rr_glidepath_ypp_summary(self, years_per_port, ports_rt, annuity_pr, target_wage, conf=0.9):
+        '''
+        Estimate glidepath receiving vector of years_per_port(ypp) as input.
+        '''
+        ports_ind = ports_onehot(n_times=ports_rt.shape[0], years_per_port=np.round(years_per_port))
+        acum_savings = np.zeros(ports_rt.shape[1])
+        for i in range(ports_rt.shape[1]):
+            rets = np.sum(ports_rt[:,i,:]*ports_ind, axis=1)
+            fwd_rets = np.sum(rets) - np.array([0]+list(np.cumsum(rets)[:-1]))
+            acum_savings[i] = np.sum(self.contrib*np.exp(fwd_rets))
+            replace_rate_dist = acum_savings*annuity_pr/target_wage
+            min_rr = np.quantile(replace_rate_dist, 1-conf)
+        return dict(years_per_port=years_per_port, ports_ind=ports_ind, acum_savings=acum_savings,replace_rate_dist=replace_rate_dist, min_rr=min_rr)
+
     def rr_glidepath_summary(self, params, ports_rt, annuity_pr,target_wage, conf=0.9):
         n_ports = ports_rt.shape[2]
         time_props = (np.arange(n_ports+1)/n_ports)[1:]
