@@ -4,7 +4,9 @@ from sklearn.preprocessing import MinMaxScaler
 from timeit import default_timer as timer
 import datetime as dt
 from dateutil.relativedelta import relativedelta
-import os, re, csv, joblib
+import os, re, csv, joblib, json, io
+from scipy import stats
+from PyCondorInvestmentAnalytics.utils import extract_data_s3select
 
 def save_dfs(dfs, filename='dataframe.csv'):
     dfs[0].to_csv(filename)
@@ -1353,24 +1355,132 @@ def times_to_labels(times):
         labels[pos_d] = [str(int(np.round(times[i]*365))) + 'D' for i in np.where(pos_d)[0]]
     return list(labels)
 
-def predict_fase(db, s3client, ts, model_names, root='', fillna_method = 'ffill',
-                 file_name = 'series_list_raven.csv', bucket = 'suraraven'):
-    ec_pred = np.zeros((len(ts),4))
-    for i in range(len(ts)):
-        time_steps = ts[i]
-        if fillna_method == 'ffill':
-            model_features = db.fetch_model_data(model_names[i], s3client, file_name = file_name, bucket = bucket).iloc[:, 1:].fillna(method = 'ffill').dropna()
-        elif fillna_method == 'zeros':
-            model_features = db.fetch_model_data(model_names[i], s3client, file_name = file_name, bucket = bucket).iloc[:, 1:].fillna(0).dropna()
-        model = joblib.load(root + model_names[i] + '.joblib')
-        ec_pred[i, :] = 100 * sklearn_model_forecast(model, model_features, predict_proba = True, custom_predict_prob=False).iloc[-1, :]
-        #try:
-        #    ec_pred[i, :] = 100 * sklearn_model_forecast(model, model_features, predict_proba = True, custom_predict_prob=True).iloc[-1, :]
-        #except:
-        #    ec_pred[i, :] = 100 * sklearn_model_forecast(model, model_features, predict_proba = True, custom_predict_prob=False).iloc[-1, :]
-    ec_pred_df = pd.DataFrame(ec_pred, index=times_to_labels(ts/12), columns = np.arange(1, 5))
+def tpot_data_prep(model_name, s3client, file_series='output/series_list_raven/series_list_raven.csv', bucket='condor-sura-qa', model_bucket='condor-ml'):
+    """
+    Fetches data required to make predictions or to train/re-train TPOT model from raven class.
+
+    Parameters
+    -- model_name - model_name from nombre_modelo column in modelo table from raven_db
+    -- s3client
+    -- file_series - Key s3 object with data
+    -- bucket - Bucket name
+    -- data_dict - dictionary with series. Keys must be with the following structure: ticker | field
+
+    Returns
+    -- Pandas dataframe
+    """
+
+    model_meta = json.loads(s3client.get_object(Bucket=model_bucket, Key=f"metadata/{model_name}.json")['Body'].read().decode('utf-8'))
+
+    lag_all_vars, lag_target = model_meta['lag_all'], model_meta['lag_target']
+    target, features = model_meta['target'], [x['ticker_campo'] for x in model_meta['features']]
+
+    vars = target + features
+    raven_sliced = extract_data_s3select(s3client, vars, file_name = file_series, bucket_name = bucket)
+    raven_concat = pd.DataFrame(raven_sliced)
+    if lag_target:
+        raven_concat = pd.concat([raven_concat, raven_concat[target]], axis=1)
+
+    ticker_campo = pd.read_csv(io.BytesIO(s3client.get_object(Bucket=model_bucket, Key='static/ticker_campo.csv')['Body'].read()), index_col = ['ticker_campo'], encoding='latin-1')
+    shift_vals = ticker_campo[ticker_campo.index.isin(raven_concat.columns)]['shift'].to_dict()
+
+    for col in raven_concat.columns:
+        raven_concat[col] = raven_concat[col].shift(shift_vals[col])
+
+    output_df = pd.DataFrame(index = raven_concat.index)
+    req_data = pd.DataFrame(model_meta['features']).set_index('ticker_campo')
+
+    for j in range(raven_concat.shape[1]):
+        col = str(raven_concat.columns[j])
+        if j == 0:
+            timestep = model_meta['timestep']
+            period = model_meta['periodo']
+            if model_meta['transformacion'] != 'none':
+                if model_meta['transformacion'] == 'pct_change':
+                    if model_meta['output_type'] == 'mode':
+                        output_df[col + ' t+ ' + str(timestep)] = raven_concat.iloc[:, j].pct_change(period).rolling(window = timestep).apply(lambda x: stats.mode(x)[0]).shift(-timestep)
+                    elif model_meta['output_type'] == 'mean':
+                        output_df[col + ' t+ ' + str(timestep)] = raven_concat.iloc[:, j].pct_change(period).rolling(window = timestep).apply(lambda x: np.mean(x)).shift(-timestep)
+                    elif model_meta['output_type'] == 'simple':
+                        output_df[col + ' t+ ' + str(timestep)] = raven_concat.iloc[:, j].pct_change(period).shift(-timestep)
+                    elif model_meta['output_type'] == 'jurik_ma':
+                        output_df[col + ' t+ ' + str(timestep)] = jurik(raven_concat.iloc[:, j].pct_change(period).rolling(window = timestep).apply(lambda x: np.mean(x)).shift(-timestep))
+                elif model_meta['transformacion'] == 'diff':
+                    if model_meta['output_type'] == 'mode':
+                        output_df[col + ' t+ ' + str(timestep)] = raven_concat.iloc[:, j].diff(period).rolling(window = timestep).apply(lambda x: stats.mode(x)[0]).shift(-timestep)
+                    elif model_meta['output_type'] == 'mean':
+                        output_df[col + ' t+ ' + str(timestep)] = raven_concat.iloc[:, j].diff(period).rolling(window = timestep).apply(lambda x: np.mean(x)).shift(-timestep)
+                    elif model_meta['output_type'] == 'simple':
+                        output_df[col + ' t+ ' + str(timestep)] = raven_concat.iloc[:, j].diff(period).shift(-timestep)
+                    elif model_meta['output_type'] == 'jurik_ma':
+                        output_df[col + ' t+ ' + str(timestep)] = jurik(raven_concat.iloc[:, j].diff(period).rolling(window = timestep).apply(lambda x: np.mean(x)).shift(-timestep))
+            else:
+                if model_meta['output_type'] == 'mode':
+                    output_df[col + ' t+ ' + str(timestep)] = raven_concat.iloc[:, j].rolling(window = timestep).apply(lambda x: stats.mode(x)[0]).shift(-timestep)
+                elif model_meta['output_type'] == 'mean':
+                    output_df[col + ' t+ ' + str(timestep)] = raven_concat.iloc[:, j].rolling(window = timestep).apply(lambda x: np.mean(x)).shift(-timestep)
+                elif model_meta['output_type'] == 'simple':
+                    output_df[col + ' t+ ' + str(timestep)] = raven_concat.iloc[:, j].shift(-timestep)
+                elif model_meta['output_type'] == 'jurik_ma':
+                    output_df[col + ' t+ ' + str(timestep)] = jurik(raven_concat.iloc[:, j].rolling(window = timestep).apply(lambda x: np.mean(x)).shift(-timestep))
+        else:
+            req_data_tick = req_data.loc[[col]]
+            for i in range(len(req_data_tick)):
+                req_data_temp = req_data_tick.iloc[i, :]
+                period_ticker = req_data_temp['periodo']
+                if req_data_temp['lag_all'] == 1:
+                    if req_data_temp['transformacion'] == 'pct_change':
+                        for k in range(1, req_data_temp['lag']):
+                            output_df[col + ' t- ' + str(k)] = raven_concat.iloc[:, j].pct_change(period_ticker).shift(k)
+                        output_df[col] = raven_concat.iloc[:, j].pct_change(period_ticker)
+                    elif req_data_temp['transformacion'] == 'diff':
+                        for k in range(1, req_data_temp['lag']):
+                            output_df[col + ' t- ' + str(k)] = raven_concat.iloc[:, j].diff(period_ticker).shift(k)
+                        output_df[col] = raven_concat.iloc[:, j].diff(period_ticker)
+                    elif req_data_temp['transformacion'] == 'none':
+                        for k in range(1, req_data_temp['lag']):
+                            output_df[col + ' t- ' + str(k)] = raven_concat.iloc[:, j].shift(k)
+                    elif req_data_temp['transformacion'] == 'rolling_mean':
+                        for k in range(1, req_data_temp['lag']):
+                            output_df[col + ' rolling_mean t- ' + str(k)] = raven_concat.iloc[:, j].rolling(period_ticker).mean().shift(k)
+                        output_df[col] = raven_concat.iloc[:, j].rolling(period_ticker).mean()
+                else:
+                    if req_data_temp['transformacion'] == 'pct_change':
+                        output_df[col + ' t- ' + str(req_data_temp['lag'])] = raven_concat.iloc[:, j].pct_change(period_ticker).shift(req_data_temp['lag'])
+                    elif req_data_temp['transformacion'] == 'diff':
+                        output_df[col + ' t- ' + str(req_data_temp['lag'])] = raven_concat.iloc[:, j].diff(period_ticker).shift(req_data_temp['lag'])
+                    elif req_data_temp['transformacion'] == 'none':
+                        output_df[col + ' t- ' + str(req_data_temp['lag'])] = raven_concat.iloc[:, j].shift(req_data_temp['lag'])
+                    elif req_data_temp['transformacion'] == 'rolling_mean':
+                        output_df[col + ' rolling_mean t- ' + str(req_data_temp['lag'])] = raven_concat.iloc[:, j].rolling(period_ticker).mean().shift(req_data_temp['lag'])
+
+    if not lag_all_vars:
+        output_df.columns = [x.replace('t- 0', '') for x in output_df.columns]
+    else:
+        output_df = pd.concat([output_df, raven_concat], axis = 1)
+        output_df = output_df[[output_df.columns[0]] + sorted(output_df.columns[output_df.columns != output_df.columns[0]])]
+        output_df = output_df.loc[:, ~output_df.columns.duplicated()]
+
+        if not lag_target:
+            output_df = output_df.drop(labels = [raven_concat.columns[0]], axis = 1)
+
+    return output_df
+
+def predict_fase_tpot(s3client, model_id, fillna_method="zeros", file_series='output/series_list_raven/series_list_raven.csv', bucket='condor-sura-qa', model_bucket='condor-ml'):
+    '''
+    Economic fase prediction using TPOT stack
+    '''
+    if fillna_method == 'ffill':
+        model_features = tpot_data_prep(model_id, s3client, file_series=file_series, bucket=bucket, model_bucket=model_bucket).iloc[:, 1:].fillna(method = 'ffill').dropna()
+    elif fillna_method == 'zeros':
+        model_features = tpot_data_prep(model_id, s3client, file_series=file_series, bucket=bucket, model_bucket=model_bucket).iloc[:, 1:].fillna(0).dropna()
+
+    model = joblib.load(io.BytesIO(s3client.get_object(Bucket=model_bucket, Key=f"models/{model_id}.joblib")['Body'].read()))
+    ec_pred = 100 * sklearn_model_forecast(model, model_features, predict_proba = True, custom_predict_prob=False).iloc[-1, :]
+    ec_pred.index = ['Expansión', 'Desaceleración','Contracción', 'Recuperación']
     ec_pred_date = 'Fecha: ' + model_features.index[-1].strftime("%Y-%m-%d")
-    return ec_pred_df, ec_pred_date
+    return ec_pred, ec_pred_date, model, model_features
+
 
 def raven_predict(db, model_identifiers, s3client, root_models, return_last_n = 1, fillna_method = 'zeros', predict_proba = False,
                 return_shap = False, file_name = 'series_list_raven.csv', bucket = 'suraraven'):
